@@ -32,6 +32,50 @@
 
 #define LIMIT(x)  ( (x) > 0xffff ? 0xff : ( (x) <= 0xff ? 0 : ( (x) >> 8 ) ) )
 
+static int *LutYr = NULL;
+static int *LutYg = NULL;
+static int *LutYb = NULL;
+static int *LutVr = NULL;
+static int *LutVrY = NULL;
+static int *LutUb = NULL;
+static int *LutUbY = NULL;
+static int *LutRv = NULL;
+static int *LutGu = NULL;
+static int *LutGv = NULL;
+static int *LutBu = NULL;
+
+unsigned char
+RGB24_TO_Y(unsigned char r, unsigned char g, unsigned char b)
+{
+return (LutYr[(r)] + LutYg[(g)] + LutYb[(b)]);
+}
+unsigned char
+YR_TO_V(unsigned char r, unsigned char y)
+{
+return (LutVr[(r)] + LutVrY[(y)]);
+}
+unsigned char
+YB_TO_U(unsigned char b, unsigned char y)
+{
+return (LutUb[(b)] + LutUbY[(y)]);
+}
+unsigned char
+R_FROMYV(unsigned char y, unsigned char v)
+{
+return CLIP((y) + LutRv[(v)]);
+}
+unsigned char
+G_FROMYUV(unsigned char y, unsigned char u, unsigned char v)
+{
+return CLIP((y) + LutGu[(u)] + LutGv[(v)]);
+}
+unsigned char
+B_FROMYU(unsigned char y, unsigned char u)
+{
+return CLIP((y) + LutBu[(u)]);
+}
+
+
 
 /* This is a simplistic approach. */
 static void ccvt_420p(int width, int height, const unsigned char *src, unsigned char *dst, int push) {
@@ -206,6 +250,62 @@ static void ccvt_420i(int width, int height, unsigned char *src, unsigned char *
     } /* ..for line */
 }
 
+void ccvt_uyvy(int width, int height, unsigned char *src, unsigned char *dest, int push) 
+{    
+    int R, G, B;
+    int Y1, Y2;
+    int cG, cR, cB;
+    int i;
+    for(i=height*width/2;i>0;i--) {
+        cB = ((*src - 128) * 454) >> 8;
+        cG = (*src++ - 128) * 88;
+        Y1 = *src++;
+        cR = ((*src - 128) * 359) >> 8;
+        cG = (cG + (*src++ - 128) * 183) >> 8;
+        Y2 = *src++;
+        
+        R = Y1 + cR;
+        G = Y1 + cG;
+        B = Y1 + cB;
+        
+        SAT(R);
+        SAT(G);
+        SAT(B);
+        
+        if(push == PUSH_RGB24) {               
+            *dest++ = B;
+            *dest++ = G;
+            *dest++ = R;
+        }
+        else {
+            *dest++ = R;
+            *dest++ = G;
+            *dest++ = B; 
+        }
+            
+        
+        R = Y2 + cR;
+        G = Y2 + cG;
+        B = Y2 + cB;
+        
+        SAT(R);
+        SAT(G);
+        SAT(B);
+        
+        if(push == PUSH_RGB24) {    
+            *dest++ = B;
+            *dest++ = G;
+            *dest++ = R;
+        }
+        else {
+            *dest++ = R;
+            *dest++ = G;
+            *dest++ = B;
+        }
+    }
+    
+}
+
 
 /* This is a really simplistic approach. Speedups are welcomed. */
 /* Derived this from the above 4:2:0 planar for yuyv */
@@ -280,104 +380,90 @@ static void ccvt_yuyv(int width, int height, const unsigned char *src, unsigned
     } /* ..for line */
 }
 
-/*#define SAT(c) \
- * if (c & (~255)) { if (c < 0) c = 0; else c = 255; }
- *
- * void ccvt_yuyv_rgb24_422(int width, int height, const void *src, void *dst) {
- * const unsigned char *s;
- * unsigned char *d;
- * //PIXTYPE_rgb24 *d;
- * int l, c;
- * int r, g, b, cr, cg, cb, y1, y2;
- *
- * l = height;
- * s = src;
- * d = dst;
- * while (l--) {
- * c = width >> 2;
- * while (c--) {
- * cb = ((*s - 128) * 454) >> 8;
- * cg = (*s++ - 128) * 88;
- * y1 = *s++;
- * cr = ((*s - 128) * 359) >> 8;
- * cg = (cg + (*s++ - 128) * 183) >> 8;
- * y2 = *s++;
- *
- *
- * r = y1 + cr;
- * b = y1 + cb;
- * g = y1 - cg;
- * SAT(r);
- * SAT(g);
- * SAT(b);
- *d++ = r;
- *d++ = g;
- *d++ = b;
- * r = y2 + cr;
- * b = y2 + cb;
- * g = y2 - cg;
- * SAT(r);
- * SAT(g);
- * SAT(b);
- *d++ = r;
- *d++ = g;
- *d++ = b;
- * }
- * }
- * }
- *
- * typedef struct {
- * unsigned char b;
- * unsigned char g;
- * unsigned char r;
- * } PIXTYPE_bgr24;
- *
- * void ccvt_yuyv_bgr24_422(int width, int height, const void *src, void *dst) {
- * const unsigned char *s;
- * PIXTYPE_bgr24 *d;
- * int l, c;
- * int r, g, b, cr, cg, cb, y1, y2;
- *
- * l = height;
- * s = src;
- * d = dst;
- * while (l--) {
- * c = width >> 2;
- * while (c--) {
- * cb = ((*s - 128) * 454) >> 8;
- * cg = (*s++ - 128) * 88;
- * y1 = *s++;
- * cr = ((*s - 128) * 359) >> 8;
- * cg = (cg + (*s++ - 128) * 183) >> 8;
- * y2 = *s++;
- *
- * r = y1 + cr;
- * b = y1 + cb;
- * g = y1 - cg;
- * SAT(r);
- * SAT(g);
- * SAT(b);
- * d->b = b;
- * d->g = g;
- * d->r = r;
- * d++;
- * r = y2 + cr;
- * b = y2 + cb;
- * g = y2 - cg;
- * SAT(r);
- * SAT(g);
- * SAT(b);
- * d->b = b;
- * d->g = g;
- * d->r = r;
- * d++;
- * }
- * }
- *
- * }
- */
+/*void ccvt_yuyv_rgb24_422(int width, int height, const void *src, void *dst) 
+{
+    const unsigned char *s;
+    unsigned char *d;
+//PIXTYPE_rgb24 *d;
+    int l, c;
+    int r, g, b, cr, cg, cb, y1, y2;
+    
+    l = height;
+    s = src;
+    d = dst;
+    while (l--) {
+        c = width >> 2;
+        while (c--) {
+            cb = ((*s - 128) * 454) >> 8;
+            cg = (*s++ - 128) * 88;
+            y1 = *s++;
+            cr = ((*s - 128) * 359) >> 8;
+            cg = (cg + (*s++ - 128) * 183) >> 8;
+            y2 = *s++;
+            r = y1 + cr;
+            b = y1 + cb;
+            g = y1 - cg;
+            SAT(r);
+            SAT(g);
+            SAT(b);
+            *d++ = r;
+            *d++ = g;
+            *d++ = b;
+            r = y2 + cr;
+            b = y2 + cb;
+            g = y2 - cg;
+            SAT(r);
+            SAT(g);
+            SAT(b);
+            *d++ = r;
+            *d++ = g;
+            *d++ = b;
+        }
+    }
+}*/
 
-/*taken from motion sources*/
+/*void ccvt_yuyv_bgr24_422(int width, int height, const void *src, void *dst) 
+{
+    const unsigned char *s;
+    PIXTYPE_bgr24 *d;
+    int l, c;
+    int r, g, b, cr, cg, cb, y1, y2;
+    
+    l = height;
+    s = src;
+    d = dst;
+    while (l--) {
+        c = width >> 2;
+        while (c--) {
+            cb = ((*s - 128) * 454) >> 8;
+            cg = (*s++ - 128) * 88;
+            y1 = *s++;
+            cr = ((*s - 128) * 359) >> 8;
+            cg = (cg + (*s++ - 128) * 183) >> 8;
+            y2 = *s++;
+            r = y1 + cr;
+            b = y1 + cb;
+            g = y1 - cg;
+            SAT(r);
+            SAT(g);
+            SAT(b);
+            d->b = b;
+            d->g = g;
+            d->r = r;
+            d++;
+            r = y2 + cr;
+            b = y2 + cb;
+            g = y2 - cg;
+            SAT(r);
+            SAT(g);
+            SAT(b);
+            d->b = b;
+            d->g = g;
+            d->r = r;
+            d++;
+        }
+    }
+}*/
 
 #define CLAMP(x)        ((x)<0?0:((x)>255)?255:(x))
 
@@ -640,6 +726,21 @@ void mjpegtoyuv420p(const unsigned char *src, unsigned char *dst,
     free(yuv[2]);
 }
 
+void ccvt_yuyv_rgb24_422(int width, int height, const void *src, void *dst) {
+    ccvt_uyvy(width, height, (unsigned char *)src, (unsigned char *)dst, PUSH_RGB24);
+}
+
+void ccvt_yuyv_bgr24_422(int width, int height, const void *src, void *dst) {
+    ccvt_uyvy(width, height, (unsigned char *)src, (unsigned char *)dst, PUSH_BGR24);
+}
+
+void ccvt_uyvy_rgb24(int width, int height, const void *src, void *dst) {
+    ccvt_uyvy(width, height, (unsigned char *)src, (unsigned char *)dst, PUSH_RGB24);
+}
+
+void ccvt_uyvy_bgr24(int width, int height, const void *src, void *dst) {
+    ccvt_uyvy(width, height, (unsigned char *)src, (unsigned char *)dst, PUSH_BGR24);
+}
 
 void ccvt_yuyv_rgb24(int width, int height, const void *src, void *dst) {
     ccvt_yuyv(width, height, (const unsigned char *)src, (unsigned char *)dst, PUSH_RGB24);
