@@ -37,13 +37,15 @@
 #include <sys/ioctl.h>
 #include <asm/types.h>          /* for videodev2.h */
 #include <linux/videodev2.h>
+#include <wx/mstream.h>
+#include <wx/image.h>
+#include <iostream>
 
 VideoForLinux2::VideoForLinux2()
 {
     dev_name = Setting::GetInstance()->GetDeviceFile();
     buffers = NULL;
     n_buffers = 0;
-    first = true;
     controlsSetted = false;
     pixelFormat = 0;
     isMapped = false;       
@@ -51,18 +53,7 @@ VideoForLinux2::VideoForLinux2()
    
 VideoForLinux2::~VideoForLinux2()
 {
-    unsigned int i;
-    enum v4l2_buf_type type;
-    type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
-
-    if (ioctl (fd, VIDIOC_STREAMOFF, &type) == -1)
-        perror("VIDIOC_STREAMOFF");
-    
-     for (i = 0; i < n_buffers; ++i)
-         if (munmap (buffers[i].start, buffers[i].length) == -1)
-             perror("munmap");
-    
-    free(buffers);
+    closeDevice();
 }
 
 void VideoForLinux2::closeDevice()
@@ -71,7 +62,6 @@ void VideoForLinux2::closeDevice()
     enum v4l2_buf_type type;
     type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
 
-    first = true;
     if (ioctl (fd, VIDIOC_STREAMOFF, &type) == -1)
         perror("VIDIOC_STREAMOFF");
     
@@ -91,6 +81,7 @@ bool VideoForLinux2::mMap(int w, int h) {
     if(!controlsSetted)
         setControls();
     
+    memset(&fmt, 0, sizeof(struct v4l2_format));
     memset(&cropcap, 0, sizeof(struct v4l2_cropcap));
     cropcap.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
     
@@ -111,12 +102,8 @@ bool VideoForLinux2::mMap(int w, int h) {
     }
     else {
                 /* Errors ignored. */
-    }
-    
-    memset(&fmt, 0, sizeof(struct v4l2_format));
-    fmt.type                = V4L2_BUF_TYPE_VIDEO_CAPTURE;
-    fmt.fmt.pix.width       = w;
-    fmt.fmt.pix.height      = h;
+    } 
+
     
     if(frameFormat==VIDEO_PALETTE_RGB24) {
         fmt.fmt.pix.pixelformat = V4L2_PIX_FMT_BGR24; 
@@ -133,6 +120,16 @@ bool VideoForLinux2::mMap(int w, int h) {
         pixelFormat = V4L2_PIX_FMT_YUV420;
         std::cout<<"Using V4L2_PIX_FMT_YUV420 pixel format"<<std::endl;
     }
+    else if(frameFormat==VIDEO_PALETTE_JPEG) {
+        fmt.fmt.pix.pixelformat = V4L2_PIX_FMT_JPEG;
+        pixelFormat = V4L2_PIX_FMT_JPEG;
+        std::cout<<"Using V4L2_PIX_FMT_JPEG pixel format"<<std::endl;
+    }
+    else if(frameFormat==VIDEO_PALETTE_MJPEG) {
+        fmt.fmt.pix.pixelformat = V4L2_PIX_FMT_MJPEG;
+        pixelFormat = V4L2_PIX_FMT_MJPEG;
+        std::cout<<"Using V4L2_PIX_FMT_MJPEG pixel format"<<std::endl;
+    }
     else { //AUTO or YUV420
         struct v4l2_fmtdesc argp;
         memset(&argp, 0, sizeof(struct v4l2_fmtdesc));
@@ -151,60 +148,46 @@ bool VideoForLinux2::mMap(int w, int h) {
                     fmt.fmt.pix.pixelformat = V4L2_PIX_FMT_YUV420;
                     pixelFormat = V4L2_PIX_FMT_YUV420;
                     std::cout<<"Found V4L2_PIX_FMT_YUV420 pixel format"<<std::endl;
+                    break;
                 }
                 else if(argp.pixelformat == V4L2_PIX_FMT_YUYV) {
                     fmt.fmt.pix.pixelformat = V4L2_PIX_FMT_YUYV;
                     pixelFormat = V4L2_PIX_FMT_YUYV;
                     std::cout<<"Found V4L2_PIX_FMT_YUYV pixel format"<<std::endl;
                 }
+                else if(argp.pixelformat == V4L2_PIX_FMT_JPEG ) {
+                    wxInitAllImageHandlers();
+                    fmt.fmt.pix.pixelformat = V4L2_PIX_FMT_JPEG;
+                    pixelFormat = V4L2_PIX_FMT_JPEG;
+                    std::cout<<"Found V4L2_PIX_FMT_JPEG pixel format"<<std::endl;                     
+                }
                 else if(argp.pixelformat == V4L2_PIX_FMT_MJPEG) {
                     fmt.fmt.pix.pixelformat = V4L2_PIX_FMT_MJPEG;
                     pixelFormat = V4L2_PIX_FMT_MJPEG;
-                    std::cout<<"Found V4L2_PIX_FMT_MJPEG pixel format"<<std::endl;
+                    std::cout<<"Found V4L2_PIX_FMT_MJPEG pixel format"<<std::endl;                     
                 }
                 else if(argp.pixelformat == V4L2_PIX_FMT_SN9C10X) {
                     fmt.fmt.pix.pixelformat = V4L2_PIX_FMT_SN9C10X;
                     pixelFormat = V4L2_PIX_FMT_SN9C10X;
-                    std::cout<<"Found V4L2_PIX_FMT_SN9C10X pixel format"<<std::endl;
+                    std::cout<<"Found V4L2_PIX_FMT_SN9C10X pixel format"<<std::endl;                    
                 }
                 else if(argp.pixelformat == V4L2_PIX_FMT_SBGGR8) {
                     fmt.fmt.pix.pixelformat = V4L2_PIX_FMT_SBGGR8;
                     pixelFormat = V4L2_PIX_FMT_SBGGR8;
-                    std::cout<<"Found V4L2_PIX_FMT_SBGGR8 pixel format"<<std::endl;
+                    std::cout<<"Found V4L2_PIX_FMT_SBGGR8 pixel format"<<std::endl;                    
                 }
                 else if(argp.pixelformat == V4L2_PIX_FMT_BGR24) {
                     fmt.fmt.pix.pixelformat = V4L2_PIX_FMT_BGR24;
                     pixelFormat = V4L2_PIX_FMT_BGR24;
                     std::cout<<"Found V4L2_PIX_FMT_BGR24 pixel format"<<std::endl;
+                    break;
                 }
-                
-                /*RGB packed formats -- there are not supported yet*/
-                /*
-                else if(argp.pixelformat == V4L2_PIX_FMT_RGB444) {
-                    fmt.fmt.pix.pixelformat = V4L2_PIX_FMT_RGB444;
-                    //pixelFormat = V4L2_PIX_FMT_RGB444;
-                    std::cout<<"Found V4L2_PIX_FMT_RGB444 pixel format"<<std::endl;
-                    std::cout<<"This pixel format is not yet supported"<<std::endl;
-                }
-                else if(argp.pixelformat == V4L2_PIX_FMT_RGB565) {
-                    fmt.fmt.pix.pixelformat = V4L2_PIX_FMT_RGB565;
-                    //pixelFormat = V4L2_PIX_FMT_RGB565;
-                    std::cout<<"Found V4L2_PIX_FMT_RGB565 pixel format"<<std::endl;
-                    std::cout<<"This pixel format is not yet supported"<<std::endl;
-                }
-                else if(argp.pixelformat == V4L2_PIX_FMT_RGB555X) {
-                    fmt.fmt.pix.pixelformat = V4L2_PIX_FMT_RGB555X;
-                    //pixelFormat = V4L2_PIX_FMT_RGB555X;
-                    std::cout<<"FoundV4L2_PIX_FMT_RGB555X pixel format"<<std::endl;
-                    std::cout<<"This pixel format is not yet supported"<<std::endl;
-                }
-                else if(argp.pixelformat == V4L2_PIX_FMT_RGB565X) {
-                    fmt.fmt.pix.pixelformat = V4L2_PIX_FMT_RGB565X;
-                    //pixelFormat = V4L2_PIX_FMT_RGB565X;
-                    std::cout<<"Found V4L2_PIX_FMT_RGB565X pixel format"<<std::endl;
-                    std::cout<<"This pixel format is not yet supported"<<std::endl;
-                }  
-                 */              
+                else if(argp.pixelformat == V4L2_PIX_FMT_UYVY) {
+                    fmt.fmt.pix.pixelformat = V4L2_PIX_FMT_UYVY;
+                    pixelFormat = V4L2_PIX_FMT_UYVY;
+                    std::cout<<"Found V4L2_PIX_FMT_UYVY pixel format"<<std::endl;
+                    break;
+                }                         
             }                
         }
     }
@@ -212,18 +195,32 @@ bool VideoForLinux2::mMap(int w, int h) {
     if(!pixelFormat) {
         std::cout<<"Could not determine pixel format or pixel format not yet supported"<<std::endl;
         return false;
+    }    
+        
+    fmt.type                = V4L2_BUF_TYPE_VIDEO_CAPTURE;
+    fmt.fmt.pix.width       = w;
+    fmt.fmt.pix.height      = h;
+    fmt.fmt.pix.field       = V4L2_FIELD_INTERLACED;
+    
+    /*enum v4l2_buf_type type;
+    type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
+    if (ioctl (fd, VIDIOC_STREAMOFF, &type) == -1)
+        perror("VIDIOC_STREAMOFF"); 
+     */
+
+    if (isMapped) {
+        closeDevice();
+        close(fd);
+
+        std::string device = Setting::GetInstance()->GetDeviceFile();
+        fd = open(device.c_str(), O_RDWR); // open video device
     }
-    
-    fmt.fmt.pix.field = V4L2_FIELD_INTERLACED;
-    
+
     if (ioctl (fd, VIDIOC_S_FMT, &fmt) == -1) {
         perror("VIDIOC_S_FMT");
         return false;
     }
     
-    if(isMapped)
-	closeDevice();
-    
     /* Buggy driver paranoia. */
     min = fmt.fmt.pix.width * 2;
     if (fmt.fmt.pix.bytesperline < min)
@@ -242,6 +239,7 @@ bool VideoForLinux2::mMap(int w, int h) {
 bool VideoForLinux2::init_mmap()
 {
     struct v4l2_requestbuffers req;
+    enum v4l2_buf_type type;
 
     memset(&req, 0, sizeof(struct v4l2_requestbuffers));
     
@@ -307,6 +305,28 @@ bool VideoForLinux2::init_mmap()
             return false;
         }
     }
+
+    for (n_buffers = 0; n_buffers < req.count; ++n_buffers) {
+        struct v4l2_buffer buf;
+        memset(&buf, 0, sizeof(struct v4l2_buffer));
+        
+        buf.type        = V4L2_BUF_TYPE_VIDEO_CAPTURE;
+        buf.memory      = V4L2_MEMORY_MMAP;
+        buf.index       = n_buffers;
+        
+        if (ioctl(fd, VIDIOC_QBUF, &buf) == -1) {
+            perror("VIDIOC_QBUF");
+            return false;
+        }
+    }
+        
+    type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
+    
+    if (ioctl(fd, VIDIOC_STREAMON, &type) == -1) {
+        perror("VIDIOC_STREAMON");
+        return false;
+    }    
+    
     isMapped = true;
     return true;
 }
@@ -314,7 +334,7 @@ bool VideoForLinux2::init_mmap()
 bool VideoForLinux2::getFrame(char **buffer)
 {
     unsigned int i;
-    enum v4l2_buf_type type;
+    
     struct v4l2_buffer buf;
     
     if(!isMapped) {
@@ -322,48 +342,26 @@ bool VideoForLinux2::getFrame(char **buffer)
         return false;
     }
     
-    if(first) {
-        for (i = 0; i < n_buffers; ++i) {
-            memset(&buf, 0, sizeof(struct v4l2_buffer));
-
-            buf.type        = V4L2_BUF_TYPE_VIDEO_CAPTURE;
-            buf.memory      = V4L2_MEMORY_MMAP;
-            buf.index       = i;
-
-            if (ioctl (fd, VIDIOC_QBUF, &buf) == -1) {
-                perror("VIDIOC_QBUF");
-                return false;
-            }
-        }
-
-        type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
-
-        if (ioctl (fd, VIDIOC_STREAMON, &type) == -1) {
-            perror("VIDIOC_STREAMON");
-            return false;
-        }
-    }
-    
-    first = false;
     memset(&buf, 0, sizeof(struct v4l2_buffer));
     
     buf.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
     buf.memory = V4L2_MEMORY_MMAP;
+    //buf.index  = n_buffers;
 
-    if (ioctl (fd, VIDIOC_DQBUF, &buf) == -1) {
+    if (ioctl(fd, VIDIOC_DQBUF, &buf) == -1) {
         switch (errno) {
             case EAGAIN:
                 return false;
                 
-                case EIO:
-                        /* Could ignore EIO, see spec. */
-
-                        /* fall through */
-
-                default:
-                    perror("VIDIOC_DQBUF");
-                    return false;
-                }
+            case EIO:
+                /* Could ignore EIO, see spec. */
+                
+                /* fall through */
+                
+            default:
+                perror("VIDIOC_DQBUF");
+                return false;
+        }
     }
 
     assert(buf.index < n_buffers);       
@@ -376,21 +374,53 @@ bool VideoForLinux2::getFrame(char **buffer)
         else
             ccvt_420p_rgb24( width, height, ( const unsigned char* ) buffers[buf.index].start, ( unsigned char* ) *buffer );             
     }
-    else if(pixelFormat == V4L2_PIX_FMT_YUYV) {        
+    else if(pixelFormat == V4L2_PIX_FMT_UYVY) {
+        if(adjustColors)
+            ccvt_uyvy_bgr24( width, height, ( const unsigned char* ) buffers[buf.index].start, ( unsigned char* ) *buffer );
+        else
+            ccvt_uyvy_rgb24( width, height, ( const unsigned char* ) buffers[buf.index].start, ( unsigned char* ) *buffer );             
+    }
+    else if(pixelFormat == V4L2_PIX_FMT_YUYV) {
         if(adjustColors)
             ccvt_yuyv_bgr24( width, height, ( const unsigned char* ) buffers[buf.index].start, ( unsigned char* ) *buffer );
         else
-            ccvt_yuyv_rgb24( width, height, ( const unsigned char* ) buffers[buf.index].start, ( unsigned char* ) *buffer );        
+            ccvt_yuyv_rgb24( width, height, ( const unsigned char* ) buffers[buf.index].start, ( unsigned char* ) *buffer );
     }
-    else if(pixelFormat == V4L2_PIX_FMT_MJPEG) {  
-        unsigned char *tmp_buffer = (unsigned char *)malloc(width*height*3/2);
-        mjpegtoyuv420p(( const unsigned char* ) buffers[buf.index].start, tmp_buffer, width, height, buf.bytesused);
-        if(adjustColors)
-            ccvt_420p_bgr24( width, height, tmp_buffer, ( unsigned char* ) *buffer );
-        else
-            ccvt_420p_rgb24( width, height, tmp_buffer, ( unsigned char* ) *buffer );    
-        free (tmp_buffer);
+    else if(pixelFormat == V4L2_PIX_FMT_MJPEG || pixelFormat == V4L2_PIX_FMT_JPEG) {
+        //std::cout<<"Using mjpeg..."<<std::endl;
+        if (buf.bytesused <= HEADERFRAME1) {
+            // Prevent crash on empty image
+            std::cout << "Ignoring empty buffer ..." <<std::endl;
+        }
+        else {
+            unsigned char *tmp_buffer = (unsigned char *)malloc(width*height*3/2);
+            mjpegtoyuv420p(( const unsigned char* ) buffers[buf.index].start, tmp_buffer, width, height, buf.length);
+            if(adjustColors)
+                ccvt_420p_bgr24( width, height, tmp_buffer, ( unsigned char* ) *buffer );
+            else
+                ccvt_420p_rgb24( width, height, tmp_buffer, ( unsigned char* ) *buffer );
+            free (tmp_buffer);
+        }
     }
+    /*else if(pixelFormat == V4L2_PIX_FMT_JPEG || pixelFormat == V4L2_PIX_FMT_MJPEG) {
+
+        if (buf.bytesused <= HEADERFRAME1) {
+            // Prevent crash on empty image
+            std::cout << "Ignoring empty buffer ..." <<std::endl;
+        } else {
+            wxMemoryInputStream in(buffers[buf.index].start, buf.bytesused);
+            wxImage frame;
+            if (!frame.LoadFile(in, wxBITMAP_TYPE_JPEG)) {
+                std::cout << "Error reading jpeg frame" << std::endl;
+                delete *buffer;
+                *buffer = NULL;
+                return false;
+            }
+            memcpy(*buffer, frame.GetData(), width * height * 3);
+            if (adjustColors)
+                convert2bgr((unsigned char*) *buffer, width * height * 3);
+        }
+    }*/
     else if(pixelFormat == V4L2_PIX_FMT_SN9C10X) {  
         unsigned char *tmp_buffer = (unsigned char *)malloc(width*height*4);
         sonix_decompress(tmp_buffer, ( unsigned char* ) buffers[buf.index].start, width, height);
@@ -409,7 +439,7 @@ bool VideoForLinux2::getFrame(char **buffer)
         std::cout<<"Unknown pixel format"<<std::endl;
         return false;
     }   
-
+    
     if (ioctl (fd, VIDIOC_QBUF, &buf) == -1) {
         perror("VIDIOC_QBUF");
         return false;
@@ -418,6 +448,53 @@ bool VideoForLinux2::getFrame(char **buffer)
     return true;
 }
 
+bool VideoForLinux2::setVideoStd(video_std std) 
+{
+    struct v4l2_input input;
+    v4l2_std_id std_id;
+
+    memset(&input, 0, sizeof (input));
+
+    if (-1 == ioctl(fd, VIDIOC_G_INPUT, &input.index)) {
+        perror("VIDIOC_G_INPUT");
+        return false;
+    }
+
+    if (-1 == ioctl(fd, VIDIOC_ENUMINPUT, &input)) {
+        perror("VIDIOC_ENUM_INPUT");
+        return false;
+    }
+    
+    int mode;
+    if(std == MODE_PAL) {
+        std_id = V4L2_STD_PAL;
+        if (0 == (input.std & V4L2_STD_PAL)) {
+            perror("Oops. PAL is not supported");
+            return false;
+        }
+    }
+    else if(std == MODE_NTSC) {
+        std_id = V4L2_STD_NTSC;
+        if (0 == (input.std & V4L2_STD_NTSC)) {
+            perror("Oops. NTSC is not supported");
+            return false;
+        }
+    }
+    else {
+        std_id = V4L2_STD_SECAM;
+        if (0 == (input.std & V4L2_STD_SECAM)) {
+            perror("Oops. SECAM is not supported");
+            return false;
+        }        
+    }
+
+    if (-1 == ioctl(fd, VIDIOC_S_STD, &std_id)) {
+        perror("VIDIOC_S_STD");
+        return false;
+    }
+    return true;
+}
+
 bool VideoForLinux2::setResolution(unsigned int width, unsigned int height, unsigned int frameRate)
 {   
     frameRate=0; /*avoiding warning*/
