@@ -31,6 +31,8 @@
 #include <wx/msgdlg.h>
 #include <wx/aboutdlg.h>
 #include <wx/thread.h>
+
+#include "device.h"
 #include <iostream>
 #include <sstream>
 #include "wxcam.h"
@@ -42,6 +44,7 @@
 #include "resolution.h"
 #include "configure.h"
 #include "frame.h"
+#include "vidiostd.h"
 
 using namespace std;
 
@@ -53,6 +56,7 @@ using namespace std;
 #include "record.xpm"
 #include "rgb.xpm"
 #include "resize.xpm"
+#include "tv.xpm"
 
 const int ID_TOOLBAR = 500;
 
@@ -63,6 +67,7 @@ EVT_MENU( Menu_Info_Capabilities, wxCam::OnShowCapabilities )
 EVT_MENU( wxID_SETUP, wxCam::OnConfigure )
 EVT_MENU( Menu_Settings_Resolution, wxCam::OnSettingResolution )
 EVT_MENU( Menu_Settings_Picture, wxCam::OnPictureFunctions )
+EVT_MENU( Menu_Settings_Video_Std, wxCam::OnSettingVideoStd )
 EVT_MENU( Menu_Video_Snapshot, wxCam::OnTakeSnapshot )
 EVT_MENU( Menu_Video_Recording, wxCam::OnRecording )
 EVT_MENU( Display_FullScreen, wxCam::OnFullScreen )
@@ -76,6 +81,7 @@ EVT_MENU( Deriche_Filter, wxCam::OnSettingDeriche )
 EVT_MENU( Noise_Filter, wxCam::OnSettingNoise )
 EVT_MENU( Sharpen_Filter, wxCam::OnSettingSharpen )
 EVT_MENU( Edge_Filter, wxCam::OnSettingEdge )
+EVT_MENU( Laplacian_Filter, wxCam::OnSettingLaplacian )
 EVT_MENU( DateTime_Filter, wxCam::OnSettingDateTime )
 EVT_MENU( Menu_MovementDetection, wxCam::OnMovementDetection )
 EVT_MENU( Menu_ShowMDAreas, wxCam::OnShowMDAreas )
@@ -119,7 +125,6 @@ wxCam::wxCam( const wxString& title )
         res = "0x0";
     }
     frame = new Frame( res, this, panel, -1, wxPoint( 0, 0 ), GetClientSize() );    
-    adjustSize(res);
     
     if ( isFirstExecution && frame->isOpen() ) {   
         Setting::GetInstance()->SetAlreadyOpen();
@@ -136,6 +141,11 @@ wxCam::wxCam( const wxString& title )
         }*/
         //else: RGB24 is the default setting
     }
+    adjustSize(/*res*/Setting::GetInstance()->GetResolution());
+    
+    if(Setting::GetInstance()->GetVideoStd() != NONE) {
+        cam.setVideoStd(Setting::GetInstance()->GetVideoStd());
+    }
     
     if(frame->isOpen())         
         frame->Started();    
@@ -161,6 +171,10 @@ wxCam::wxCam( const wxString& title )
     wxMenuItem *resizeItem = new wxMenuItem( menuSettings, Menu_Settings_Resolution, _( "&Frame size\tCtrl-F" ), _( "Choose resolution and frame rate" ) );
     resizeItem->SetBitmap( resizeRbg );
     menuSettings->Append( resizeItem );
+    wxBitmap tv( tv_xpm );
+    wxMenuItem *vstdItem = new wxMenuItem( menuSettings, Menu_Settings_Video_Std, _( "&Video standard\tCtrl-V" ), _( "Choose video standard" ) );
+    vstdItem->SetBitmap( tv );
+    menuSettings->Append( vstdItem );
     menuSettings->AppendSeparator();
     menuSettings->AppendCheckItem( Display_FullScreen, _( "Full screen\tF" ) );
     
@@ -209,7 +223,9 @@ wxCam::wxCam( const wxString& title )
     menuFilters->AppendCheckItem( Sharpen_Filter, _( "&Sharpen (slow)\tAlt-S" ) );
     menuFilters->Check(Sharpen_Filter, Setting::GetInstance()->GetSharpen()); 
     menuFilters->AppendCheckItem( Edge_Filter, _( "&Edge\tAlt-E" ) );
-    menuFilters->Check(Edge_Filter, Setting::GetInstance()->GetEdge()); 
+    menuFilters->Check(Edge_Filter, Setting::GetInstance()->GetEdge());
+    menuFilters->AppendCheckItem( Laplacian_Filter, _( "&Laplacian\tAlt-L" ) );
+    menuFilters->Check(Laplacian_Filter, Setting::GetInstance()->GetLaplacian());
     menuFilters->AppendSeparator();
     menuFilters->AppendCheckItem( DateTime_Filter, _( "A&ppend datetime \tAlt-P" ) );
     menuFilters->Check(DateTime_Filter, Setting::GetInstance()->GetDateTime()); 
@@ -281,9 +297,9 @@ void wxCam::OnAbout( wxCommandEvent& WXUNUSED( event ) )
     wxAboutDialogInfo info;
     
     info.SetName( _T( "wxCam" ) );
-    info.SetVersion( wxString("1.0.1", wxConvUTF8) );
+    info.SetVersion( wxString("1.0.7", wxConvUTF8) );
     info.SetDescription( _( "A webcam application for linux" ) );
-    info.SetCopyright( _T( "(C) 2007 Marco Lorrai" ) );
+    info.SetCopyright( _T( "(C) 2010 Marco Lorrai" ) );
     info.AddDeveloper( _T( "Marco Lorrai" ) );
     info.AddDeveloper( _T( "Örjan Medhage" ) );
     // we can add a second developer
@@ -293,7 +309,7 @@ void wxCam::OnAbout( wxCommandEvent& WXUNUSED( event ) )
 //    static const wxChar *docwriters[] =
 //    {
 //        _T("Marco Lorrai"),
-//        _T("Anther writer")
+//        _T("Another Writer")
 //    };
     
     //info.SetDocWriters(wxArrayString(WXSIZEOF(docwriters), docwriters));
@@ -393,6 +409,23 @@ void wxCam::OnSettingResolution( wxCommandEvent& WXUNUSED( event ) )
         wxMessageBox( _( "Error!\nPerhaps the device hasn't been correctly open." ), _( "Setting Resolution" ), wxICON_WARNING );
 }
 
+void wxCam::OnSettingVideoStd( wxCommandEvent& WXUNUSED( event ) )
+{
+    VidioStd *std = new VidioStd( this );
+    if(std->ShowModal() == wxID_OK) {
+        video_std vm = (video_std)std->stdMode->GetSelection();
+        if (vm != NONE) {
+            if (cam.setVideoStd(vm))
+                Setting::GetInstance()->SetVideoStd(vm);
+            else
+                wxMessageBox(_("Error!\nThis video standard is not supported by your device."), _("Setting Video Standard"), wxICON_WARNING);
+        } 
+        else
+            Setting::GetInstance()->SetVideoStd(vm);
+    }
+    std->Destroy();
+}
+
 void wxCam::OnTakeSnapshot( wxCommandEvent& WXUNUSED( event ) ) 
 {
     if ( frame->isOpen() ) {
@@ -459,7 +492,7 @@ void wxCam::adjustSize(std::string res)
 
 void wxCam::fillValidResolution() 
 {
-    struct video_capability caps;
+    /*struct video_capability caps;
     int minw, minh, maxw, maxh;
     
     cam.queryCapabilities( &caps );
@@ -481,7 +514,9 @@ void wxCam::fillValidResolution()
         validResolution.Add( wxString(str.c_str(), wxConvUTF8) );
         currentw /= 2;
         currenth /= 2;
-    }
+    }*/
+    validResolution.clear();
+    cam.getResolutionList(validResolution);
 }
 
 void wxCam::OnFullScreen( wxCommandEvent& event ) 
@@ -491,9 +526,9 @@ void wxCam::OnFullScreen( wxCommandEvent& event )
             wxFULLSCREEN_NOBORDER|
             wxFULLSCREEN_NOCAPTION
             //wxFULLSCREEN_ALL
-            );
-    panel->SetSize(1600, 1200);
+            );    
     frame->setFullscreen( event.IsChecked() );
+    panel->SetSize(1920, 1440);
 }
 
 void wxCam::OnSettingAdjustColors(wxCommandEvent& event)
@@ -546,6 +581,11 @@ void wxCam::OnSettingEdge(wxCommandEvent& event)
     Setting::GetInstance()->SetEdge(event.IsChecked());
 }
 
+void wxCam::OnSettingLaplacian(wxCommandEvent& event)
+{
+    Setting::GetInstance()->SetLaplacian(event.IsChecked());
+}
+
 void wxCam::OnSettingDateTime(wxCommandEvent& event)
 {
     Setting::GetInstance()->SetDateTime(event.IsChecked());
